//-------------------------------------------------------------------------------
// DBObject
//-------------------------------------------------------------------------------

datablock StaticShapeData(DBObject)
{
   // The category variable determines where the item
   // shows up in the mission editor's creator tree.
   // Path is relative to where the script is executed from.
   category = "Restaurant";
   shapeFile = "~/data/shapes/restaurant/stove/stove.dts";
   sitAnim = "kneel";
   bitmap = "";
   noun = "thing";
   soundEat = "~/data/sound/cogmac/bite.wav";
   soundPickup = "~/data/sound/cogmac/pickup.wav";
   soundPutDown = "~/data/sound/cogmac/plate.wav";
   soundGive = "~/data/sound/cogmac/whoosh.wav";
   soundTouch = "~/data/sound/cogmac/horn.wav";
   touchRot = 0.0;
   reachDist = 2.1;
   isEmpty = false;
   isDirty = false;
   cleansDirty = false;
};

function DBObject::onAdd(%this, %obj)
{
   COGLOG_ACTION( %obj, "SPAWN" );
   COGLOG_STATECHANGE( %obj, "DELETED", "FALSE" );
   COGLOG_STATECHANGE( %obj, "ATPOS", COGLOG_POS(%obj.getPosition()) );

   if( %obj.getDataBlock().isEmpty ) COGLOG_STATECHANGE( %obj, "ISEMPTY", "TRUE" );
   else COGLOG_STATECHANGE( %obj, "ISEMPTY", "FALSE" );

   schedule( 500, 0, ReportSurface, %obj );
}

function ReportSurface( %obj )
{
   %surface = FindSurface( %obj );
   COGLOG_STATECHANGE( %obj, "ON", COGLOG_ID(%surface) );
}

function FindSurface( %obj )
{
   %pos = %obj.getPosition();
   %startPoint = VectorAdd(%pos, "0 0 1");
   %endPoint = VectorAdd(%pos, "0 0 -100");
   %searchMasks = $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType;
   %hit = ContainerRayCast (%startPoint, %endPoint, %searchMasks, %obj);
   return getWord(%hit, 0);
}

function DBObject::OnPickup( %this, %client, %user, %obj )
{
   CommandToClient( %client, 'CleanupPopup' );

   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );

   // Don't pick up something that someone is sitting on.
   %someone = %obj.getMountedObject(0);
   if( %someone != 0 )
   {
      COGLOG_ACTION( %client.player, "PICKUP", %obj );

      SetNarration( %client, "[" @ %client.player.getName() SPC "tried to pick up the" SPC %obj.getDataBlock().noun SPC and SPC %someone.getName() @ "]" );

      // Play a pickup sound.
      %soundPickup = %user.getDataBlock().soundPickup;
      PlaySound( %soundPickup, %user.getPosition() );
      return;
   }

   // Play a pickup sound.
   PlaySound( %this.soundPickup, %user.getPosition() );

   // Are we already carrying something?
   %mounted = %user.getMountedObject( 0 );
   
   // Are we picking something up from someone else?
   if( %obj.isMounted() )
   {
      %player = %obj.getObjectMount();
      %player.unmountObject( %obj );

      COGLOG_ACTION( %player, "PUTDOWN", %obj );
      COGLOG_STATECHANGE( %obj, "ATTACHEDTO", "NULL" );
   }

   // Pick up the selected object.
   %user.mountObject( %obj, 0 );

   // Drop anything we were already carrying.
   if( %mounted > 0 )
   {
      // Swap with someone.
      if( %player > 0 )
      {
         COGLOG_ACTION( %client.player, "PUTDOWN", %mounted );
         COGLOG_STATECHANGE( %mounted, "ATTACHEDTO", "NULL" );
         COGLOG_ACTION( %player, "PICKUP", %mounted );
         COGLOG_STATECHANGE( %mounted, "ATTACHEDTO", COGLOG_ID(%player) );

         %user.unmountObject( %mounted );
         %player.mountObject( %mounted, 0 );
         return;
      }

      // Drop on a surface.
      // Adjust according to object's center.
      %pos = %obj.getTransform();
      %box = %obj.getObjectBox();
      %center = %obj.getBoundingBoxCenter();
      %pos = VectorAdd(%pos, getWords(%center, 0, 1) SPC getWord(%box, 2));
      %mounted.getDataBlock().OnPutDown( %client, %user, %mounted, %obj, %pos, %obj );
   }

   COGLOG_ACTION( %client.player, "PICKUP", %obj );
   COGLOG_STATECHANGE( %obj, "ON", "NULL" );
   COGLOG_STATECHANGE( %obj, "ATTACHEDTO", COGLOG_ID(%client.player) );
}

function DBObject::OnPutDown( %this, %client, %user, %obj, %surface, %pos, %ignore )
{
   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );

   // Play a putdown sound.
   PlaySound( %this.soundPutDown, %user.getPosition() );

   %startPoint = VectorAdd(%pos, "0 0 1");
   %endPoint = VectorAdd(%pos, "0 0 -100");
   %searchMasks = $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType;


   // Don't drop an object on itself!
   if( %ignore == null ) %ignore = %obj;

   // Center the object on the bottom of its box.
   %box = %obj.getObjectBox();
   %center = %obj.getBoundingBoxCenter();

   %hit = ContainerRayCast (%startPoint, %endPoint, %searchMasks, %ignore);
   if(%hit)
   {
      // Center the object on the click pos.
      %pos = getWords( %hit, 1, 3 );
      %pos = VectorSub(%pos, getWords(%center, 0, 1) SPC getWord(%box, 2));
      %user.unmountObject( %obj );
      %Transform = %pos @ "  0 0 1 0";
      %obj.setTransform(%Transform);

      %hitObj = getWord(%hit,0);
      COGLOG_ACTION( %client.player, "PUTDOWN", %obj );
      COGLOG_STATECHANGE( %obj, "ATTACHEDTO", "NULL" );
      COGLOG_STATECHANGE( %obj, "ON", COGLOG_ID(%hitObj) );
      COGLOG_STATECHANGE( %obj, "ATPOS", COGLOG_POS(%obj.getPosition()) );

      // Clean dirty objects.
      if( %obj.getDataBlock().isDirty && %hitObj.getDataBlock().cleansDirty )
      {
         // Clean the item in a couple seconds.
         schedule( 2000, 0, CleanItem, %obj );
      }
   }

   CommandToClient( %client, 'CleanupPopup' );
}

function CleanItem( %obj )
{
   // Play a clean sound.
   %soundSpawn = "~/data/sound/cogmac/magic_bling.wav";
   PlaySound( %soundSpawn, %obj.getPosition() );

   // Delete the object.
   COGLOG_STATECHANGE( %obj, "DELETED", "TRUE" );
   %obj.delete();
}

function DBObject::OnLookAt( %this, %client, %obj )
{
   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );
   COGLOG_ACTION( %client.player, "LOOKAT", %obj );
   CommandToClient( %client, 'ShowLookAtGui', %this.bitmap );
}

function DBObject::OnGive( %this, %client, %user, %obj, %target, %pos )
{
   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );
   DBObject::HandleGive( %this, %client, %user, %obj, %target, %pos );
}

function DBObject::HandleGive( %this, %client, %user, %obj, %target, %pos )
{
   CommandToClient( %client, 'CleanupPopup' );

   if( %target.getClassName() $= "AIPlayer" )
   {
      // Play a give sound.
      PlaySound( %this.soundGive, %user.getPosition() );

      COGLOG_ACTION( %client.player, "GIVE", %obj );

      // Is receiver already carrying something?
      // If so, swap!
      %mounted = %target.getMountedObject( 0 );
      if( %mounted > 0 )
      {
         COGLOG_ACTION( %target, "PUTDOWN", %mounted );
         COGLOG_STATECHANGE( %mounted, "ATTACHEDTO", "NULL" );
         COGLOG_ACTION( %client.player, "PICKUP", %mounted );
         COGLOG_STATECHANGE( %mounted, "ATTACHEDTO", COGLOG_ID(%client.player) );

         %target.unmountObject( %mounted );
         %user.mountObject( %mounted, 0 );
      }

      COGLOG_STATECHANGE( %obj, "ATTACHEDTO", COGLOG_ID(%target) );

      // Give what we were carrying to the receiver.
      %user.unmountObject( %obj );
      %target.mountObject( %obj, 0 );
      return;
   }

   // Target is something other than a player.
   DBObject::OnPutDown( %this, %client, %user, %obj, %target, %pos, null );
}

function DBObject::OnSit( %this, %client, %user, %obj )
{
   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );

   // Already sitting on something.
   if( %user.getDataBlock().isSitting )
   {
      %mounted = %user.getObjectMount();
      %mounted.unmountObject( %user );
      COGLOG_ACTION( %client.player, "GETOFF", %mounted );
      COGLOG_STATECHANGE( %client.player, "SITTINGON", "NULL" );
   }

   // Record last standing pos.
   else {
      %user.getDataBlock().lastStandingXform = %user.getTransform();
   }

   COGLOG_ACTION( %client.player, "SITON", %obj );

   // Drop something we are holding before sitting on it.
   if( %obj.isMounted() && %obj.getObjectMount() == %user )
   {
      %user.unmountObject( %obj );
      COGLOG_STATECHANGE( %obj, "ATTACHEDTO", "NULL" );

      %startPoint = VectorAdd(%obj.getPosition(), "0 0 1");
      %endPoint = VectorAdd(%obj.getPosition(), "0 0 -100");
      %searchMasks = $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::InteriorObjectType;

      // Move object to floor.
      %hit = ContainerRayCast (%startPoint, %endPoint, %searchMasks, %obj);
      if(%hit)
      {
         %pos = getWords( %hit, 1, 3 );
         %Transform = %pos @ "  0 0 1";
         %obj.setTransform(%Transform);
         COGLOG_STATECHANGE( %obj, "ATPOS", COGLOG_POS(%obj.getPosition()) );
      }
   }

   %user.setActionThread( %obj.getDataBlock().sitAnim, false, false );
   %xform = %obj.getTransform();

   %data = %user.getDataBlock();
   %data.isSitting = true;

   // Mount to slot 0.
   %obj.mountObject( %user, 0 );
   COGLOG_STATECHANGE( %client.player, "SITTINGON", COGLOG_ID(%obj) );

   // Face the direction that the object is facing.
   %user.setTransform( %xform );
}

function DBObject::OnEat( %this, %client, %obj )
{
   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );
   COGLOG_ACTION( %client.player, "EAT", %obj );

   SetNarration( %client, "[" @ %client.player.getName() SPC "bit the" SPC %obj.getDataBlock().noun @ "]" );

   // Play an eating sound.
   PlaySound( %this.soundEat, %obj.getPosition() );
   CommandToClient( %client, 'CleanupPopup' );
}

function DBObject::OnTouch( %this, %client, %user, %obj )
{
   COGLOG_STATECHANGE( %client.player, "ATPOS", COGLOG_POS(%client.player.getPosition()) );
   COGLOG_ACTION( %client.player, "TOUCH", %obj );

   SetNarration( %client, "[" @ %client.player.getName() SPC "touched the" SPC %obj.getDataBlock().noun @ "]" );

   // Rotate the object.
   if( %this.touchRot > 0.0 ) %obj.rotateZ( %this.touchRot );

   // Play a touch sound.
   PlaySound( %this.soundTouch, %user.getPosition() );
   CommandToClient( %client, 'CleanupPopup' );
}

//-------------------------------------------------------------------------------

datablock StaticShapeData(DBCuisinart : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/cuisinart/cuisinart.dts";
   bitmap = "restaurant/data/shapes/restaurant/cuisinart/cuisinart.bmp";
   soundTouch = "~/data/sound/cogmac/grinder.wav";
   noun = "cuisinart";
};

datablock StaticShapeData(DBFoodPrep_Empty : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/food_prep/food_prep_empty.dts";
   bitmap = "restaurant/data/shapes/restaurant/food_prep/food_prep.bmp";
   soundTouch = "~/data/sound/cogmac/chop2.wav";
   isEmpty = true;
   noun = "cutting board";
};

datablock StaticShapeData(DBMicrowave : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/microwave/microwave.dts";
   bitmap = "restaurant/data/shapes/restaurant/microwave/microwave.bmp";
   soundTouch = "~/data/sound/cogmac/button_beeps.wav";
   noun = "microwave";
};

datablock StaticShapeData(DBPan : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/pan/pan.dts";
   bitmap = "restaurant/data/shapes/restaurant/pan/pan.bmp";
   soundTouch = "~/data/sound/cogmac/metal_clang.wav";
   touchRot = 0.32;
   noun = "pan";
};

datablock StaticShapeData(DBPot : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/pot/pot.dts";
   bitmap = "restaurant/data/shapes/restaurant/pot/pot.bmp";
   soundTouch = "~/data/sound/cogmac/metal_clang.wav";
   touchRot = 0.32;
   noun = "pot";
};

datablock StaticShapeData(DBWine_Empty : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/wine/wine.dts";
   bitmap = "restaurant/data/shapes/restaurant/wine/wine.bmp";
   soundTouch = "~/data/sound/cogmac/uncork.wav";
   touchRot = 0.32;
   isEmpty = true;
   isDirty = true;
   isBeverage = true;
   noun = "wine bottle";
};

datablock StaticShapeData(DBTrash_Empty : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/trash_can/trash_can_empty.dts";
   bitmap = "restaurant/data/shapes/restaurant/trash_can/trash_can.bmp";
   soundTouch = "~/data/sound/cogmac/flies.wav";
   isEmpty = true;
   noun = "trash can";
};

datablock StaticShapeData(DBFruit_Bowl : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/fruit/fruit_bowl.dts";
   bitmap = "restaurant/data/shapes/restaurant/fruit/fruit.bmp";
   soundTouch = "~/data/sound/cogmac/plate.wav";
   touchRot = 0.32;
   isEmpty = true;
   noun = "fruit bowl";
};

datablock StaticShapeData(DBBottle_Empty : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/bottle/bottle.dts";
   bitmap = "restaurant/data/shapes/restaurant/bottle/bottle.bmp";
   soundTouch = "~/data/sound/cogmac/water2.wav";
   touchRot = 0.32;
   isEmpty = true;
   isDirty = true;
   isBeverage = true;
   noun = "water bottle";
};

datablock StaticShapeData(DBBrandy_Empty : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/brandy_bottle/brandy_bottle.dts";
   bitmap = "restaurant/data/shapes/restaurant/brandy_bottle/brandy_bottle.bmp";
   soundTouch = "~/data/sound/cogmac/glass.wav";
   touchRot = 0.32;
   isEmpty = true;
   isDirty = true;
   isBeverage = true;
   noun = "brandy bottle";
};

datablock StaticShapeData(DBBlender : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/blender/blender.dts";
   bitmap = "restaurant/data/shapes/restaurant/blender/blender.bmp";
   soundTouch = "~/data/sound/cogmac/mixer2.wav";
   noun = "blender";
};

datablock StaticShapeData(DBPlate : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/plate/plate.dts";
   bitmap = "restaurant/data/shapes/restaurant/plate/plate.bmp";
   soundTouch = "~/data/sound/cogmac/plate.wav";
   touchRot = 0.32;
   isEmpty = true;
   isDirty = true;
   noun = "plate";
};

datablock StaticShapeData(DBBowl : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/soup/bowl.dts";
   bitmap = "restaurant/data/shapes/restaurant/soup/bowl.bmp";
   soundTouch = "~/data/sound/cogmac/plate.wav";
   touchRot = 0.32;
   isEmpty = true;
   isDirty = true;
   noun = "bowl";
};

datablock StaticShapeData(DBMug : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/coffee/mug.dts";
   bitmap = "restaurant/data/shapes/restaurant/coffee/mug.bmp";
   soundTouch = "~/data/sound/cogmac/plate.wav";
   touchRot = 0.32;
   isEmpty = true;
   isDirty = true;
   isBeverage = true;
   noun = "mug";
};

datablock StaticShapeData(DBTeaCup : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/tea/teacup.dts";
   bitmap = "restaurant/data/shapes/restaurant/tea/teacup.bmp";
   soundTouch = "~/data/sound/cogmac/plate.wav";
   touchRot = 0.32;
   isEmpty = true;
   isDirty = true;
   isBeverage = true;
   noun = "tea cup";
};

datablock StaticShapeData(DBVase : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/bud_vase/vase.dts";
   bitmap = "restaurant/data/shapes/restaurant/bud_vase/vase.bmp";
   soundTouch = "~/data/sound/cogmac/glass.wav";
   touchRot = 0.32;
   noun = "vase";
};

datablock StaticShapeData(DBGlass : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/glass/glass.dts";
   bitmap = "restaurant/data/shapes/restaurant/glass/glass.bmp";
   soundTouch = "~/data/sound/cogmac/glass.wav";
   touchRot = 0.32;
   isEmpty = true;
   isDirty = true;
   isBeverage = true;
   noun = "glass";
};

datablock StaticShapeData(DBWineGlass : DBObject)
{
   className = "DBObject";
   shapeFile = "~/data/shapes/restaurant/white_wine/empty_wine.dts";
   bitmap = "restaurant/data/shapes/restaurant/red_wine/wine_glass.bmp";
   soundTouch = "~/data/sound/cogmac/glass.wav";
   touchRot = 0.32;
   isEmpty = true;
   isDirty = true;
   isBeverage = true;
   noun = "wine glass";
};












