//-------------------------------------------------------------------------------
// RegionTrigger
//-------------------------------------------------------------------------------

datablock TriggerData( RegionTrigger : DefaultTrigger )
{
   // tickPeriodMS is a value is used to control how often the console
   // onTriggerTick callback is called while there are any objects
   // in the trigger. The default value is 100 MS.

   tickPeriodMS = 100;
};

function RegionTrigger::onEnterTrigger( %this, %trigger, %obj )
{
   COGLOG_EVENT( %obj, "ENTERED", %trigger );
   Parent::onEnterTrigger( %this, %trigger, %obj );
}

function RegionTrigger::onLeaveTrigger( %this, %trigger, %obj )
{
   // This method is called whenever an object leaves the %trigger
   // area, the object is passed as %obj.  The default onLeaveTrigger
   // method (in the C++ code) invokes the ::onTrigger(%trigger,0) method on
   // every object (whatever it's type) in the same group as the trigger.

   COGLOG_EVENT( %obj, "EXITED", %trigger );
   Parent::onLeaveTrigger( %this, %trigger, %obj );
}

//-------------------------------------------------------------------------------
// ExitTrigger
//-------------------------------------------------------------------------------

datablock TriggerData( ExitTrigger : DefaultTrigger )
{
   // tickPeriodMS is a value is used to control how often the console
   // onTriggerTick callback is called while there are any objects
   // in the trigger. The default value is 100 MS.

   tickPeriodMS = 100;
};

function ExitTrigger::onEnterTrigger( %this, %trigger, %obj )
{
   COGLOG_EVENT( %obj, "ENTERED", %trigger );

   // Find other player's name.
   %other = %obj.other;
   %count = ClientGroup.getCount();	
   for ( %i = 0; %i < %count; %i++ )
   {
      %client = ClientGroup.getObject( %i );
      if( %client.player.getName() !$= %obj.getName() )
      {
         %client.player.other = %obj.getName();
         %other = %client.player.getName();
         break;
      }
   }

   // Send message to client of player who entered trigger.
   for ( %i = 0; %i < %count; %i++ )
   {
      %client = ClientGroup.getObject( %i );
      if( %client.player.getName() $= %obj.getName() )
      {
         CommandToClient( %client, 'PlayerExit', %other );
         break;
      }
   }

   Parent::onEnterTrigger( %this, %trigger, %obj );
}

function ExitTrigger::onLeaveTrigger( %this, %trigger, %obj )
{
   // This method is called whenever an object leaves the %trigger
   // area, the object is passed as %obj.  The default onLeaveTrigger
   // method (in the C++ code) invokes the ::onTrigger(%trigger,0) method on
   // every object (whatever it's type) in the same group as the trigger.

   COGLOG_EVENT( %obj, "EXITED", %trigger );
   Parent::onLeaveTrigger( %this, %trigger, %obj );
}

//-------------------------------------------------------------------------------
// OrderTrigger
//-------------------------------------------------------------------------------

datablock TriggerData( OrderTrigger : DefaultTrigger )
{
   // tickPeriodMS is a value is used to control how often the console
   // onTriggerTick callback is called while there are any objects
   // in the trigger. The default value is 100 MS.

   tickPeriodMS = 100;
};

datablock TriggerData( FoodOrderTrigger : OrderTrigger )
{
   className = "OrderTrigger";
   dummy = "chef";
};

datablock TriggerData( DrinkOrderTrigger : OrderTrigger )
{
   className = "OrderTrigger";
   dummy = "bartender";
};

function OrderTrigger::onEnterTrigger( %this, %trigger, %obj )
{
   COGLOG_EVENT( %trigger.getDataBlock().dummy, "STARTLISTENING", %obj );
   COGLOG_EVENT( %obj, "ENTERED", %trigger );
   Parent::onEnterTrigger( %this, %trigger, %obj );
}

function OrderTrigger::onLeaveTrigger( %this, %trigger, %obj )
{
   // This method is called whenever an object leaves the %trigger
   // area, the object is passed as %obj.  The default onLeaveTrigger
   // method (in the C++ code) invokes the ::onTrigger(%trigger,0) method on
   // every object (whatever it's type) in the same group as the trigger.

   COGLOG_EVENT( %trigger.getDataBlock().dummy, "STOPLISTENING", %obj );
   COGLOG_EVENT( %obj, "EXITED", %trigger );
   Parent::onLeaveTrigger( %this, %trigger, %obj );
}

function OrderTrigger::ProcessOrder( %this, %trigger, %container, %text, %type, %speaker, %dummy )
{
   // Ignore a speaker who is not facing the dummy.
   if( !%speaker.isInFOV( %dummy, 180 ) ) return;

   %spawned = false;
   for( %i=0; %i < $menuItemList.nItems; %i++ )
   {
      if( IsItemReferenced( $menuItemList, %i, %text, %type ) )
      {
         %searchMasks = $TypeMasks::ItemObjectType | $TypeMasks::StaticTSObjectType | 
                        $TypeMasks::StaticShapeObjectType | $TypeMasks::TerrainObjectType | 
                        $TypeMasks::InteriorObjectType;

         // Pick a random drop position.
         %pos = %container.getRandomContainedPos();
         %end = VectorAdd(%pos, "0 0 -100");
         %scanTarg = ContainerRayCast(%pos, %end, %searchMasks);
         %drop = getWords( %scanTarg, 1, 3 );
         %surface = getWord( %scanTarg, 0 );

         // Spawn the item in about a second.
         schedule( 1000, 0, SpawnItem, $menuItemList.lstItems[%i].db, $menuItemList.lstItems[%i].type, %drop, %surface );
         %spawned = true;
      }
   }

   if( %spawned )
   {
      // Play a make sound.
      if( %type $= "food" ) %soundMake = "~/data/sound/cogmac/grill.wav";
      else %soundMake = "~/data/sound/cogmac/clink.wav";
      PlaySound( %soundMake, %dummy.getPosition() );
   }
}

function SpawnItem( %db, %type, %pos, %surface )
{
   %item = new StaticShape() { dataBlock = %db; };

   // Center the object on the bottom of its box.
   %box = %item.getObjectBox();
   %center = %item.getBoundingBoxCenter();
   %pos = VectorSub(%pos, getWords(%center, 0, 1) SPC getWord(%box, 2));
   %item.setPosition( %pos );

   COGLOG_STATECHANGE( %item, "ATPOS", COGLOG_POS(%pos) );
   COGLOG_STATECHANGE( %item, "ON", COGLOG_ID(%surface) );

   if( %type $= "food" ) %staff = "Chef";
   else %staff = "Bartender";
   SetNarration( null, "[" @ %staff SPC "put the" SPC %db.noun SPC "on the counter]" );

   // Play a spawn sound.
   %soundSpawn = "~/data/sound/cogmac/magic_bong.wav";
   PlaySound( %soundSpawn, %pos );
}








